@@ -11,7 +11,7 @@
  *		sprintf into an allocated-on-demand buffer
  *
  *
- * Portions Copyright (c) 1996-2022, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2024, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -122,8 +122,8 @@ pvsnprintf(char *buf, size_t len, const char *fmt, va_list args)
 #ifndef FRONTEND
 		elog(ERROR, "vsnprintf failed: %m with format string \"%s\"", fmt);
 #else
-		fprintf(stderr, "vsnprintf failed: %s with format string \"%s\"\n",
-				strerror(errno), fmt);
+		fprintf(stderr, "vsnprintf failed: %m with format string \"%s\"\n",
+				fmt);
 		exit(EXIT_FAILURE);
 #endif
 	}
